from masonite.contracts.UploadContract import UploadContract
from masonite.drivers.upload.BaseUploadDriver import BaseUploadDriver
from masonite.exceptions import DriverLibraryNotFound
from masonite.managers.UploadManager import UploadManager
from masonite.helpers import config
import os


class UploadMinioDriver(BaseUploadDriver, UploadContract):

    def __init__(self, upload: UploadManager):
        self.upload = upload
        self.config = config("storage")

    def store(self, fileitem, filename=None, location=None):
        """Store the file into minio server.

        Arguments:
            fileitem {cgi.Storage} -- Storage object.

        Keyword Arguments:
            location {string} -- The location on disk you would like to store the file. (default: {None})
            filename {string} -- A new file name you would like to name the file. (default: {None})

        Raises:
            DriverLibraryNotFound -- Raises when the boto3 library is not installed.

        Returns:
            string -- Returns the file name just saved.
        """
        try:
            import boto3
        except ImportError:
            raise DriverLibraryNotFound(
                'Could not find the "boto3" library. Please pip install this library by running "pip install boto3"'
            )

        driver = self.upload.driver("disk")
        driver.accept_file_types = self.accept_file_types
        driver.store(fileitem, filename=filename, location="storage/temp")
        file_location = driver.file_location

        if filename is None:
            filename = self.get_name(fileitem)

        # Check if is a valid extension
        self.validate_extension(self.get_name(fileitem))

        session = boto3.Session(
            aws_access_key_id=self.config.DRIVERS["minio"]["client"],
            aws_secret_access_key=self.config.DRIVERS["minio"]["secret"],
        )

        client = session.client(
            "s3",
            endpoint_url=self.config.DRIVERS["minio"]["endpoint"],
        )

        if location:
            location = os.path.join(location, filename)
        else:
            location = os.path.join(filename)

        client.upload_file(file_location, self.config.DRIVERS["minio"]["bucket"], location)

        return filename



from setuptools import setup

setup(
    name="minio_driver",
    version='0.0.6',
    packages=['minio_driver'],
    author="Yubaraj Shrestha",
    author_email="companion.krish@outlook.com",
    install_requires=[
        'masonite',
    ],
    description="Minio Storage Driver for Masonite",
    license="MIT",
    include_package_data=True,
    keywords=["masonite", "storage", "minio", "masonite-storage-driver"]
)



"""A MinioProvider Service Provider."""

from masonite.managers.UploadManager import UploadManager
from masonite.provider import ServiceProvider
from masonite.view import View

from . import UploadMinioDriver
from masonite.helpers import config
from masonite.helpers.static import static
from masonite import Upload


class MinioProvider(ServiceProvider):
    """Provides Services To The Service Container."""

    wsgi = False

    def register(self):
        """Register objects into the Service Container."""
        self.app.bind('UploadMinioDriver', UploadMinioDriver)

    def boot(self, manager: UploadManager, view: View):
        """Boots services required by the container."""
        self.app.bind("Upload", manager.driver(config("storage").DRIVER))
        self.app.swap(Upload, manager.driver(config("storage").DRIVER))
        view.share({"static": static})
